from baangt.base.ApiHandling import ApiHandling

class SOAPHandling(ApiHandling):
    """
    This is the Class to handle SOAP-Specific functionality of baangt. It inherits all methods and parameters from
    ApiHandling
    """
    pass

